;;;; -*- Mode: LISP; Syntax: Common-Lisp; Package: STANDARD-LISP; Base: 10 -*-

;;;; READER, Module SLCP

;;; ***************************************************************************
;;;
;;;        (c) Copyright 1985 by Lucid Inc.
;;;        Unrestricted permission is granted to copy, modify
;;;        or redistribute this file.
;;;
;;; ***************************************************************************

;;; Create a Standard Lisp readtable

;;; Programmer: Eric Benson

;;; Edit-History:
;;;
;;; Created: Monday the eighth of July, 1985; 8:12:40 pm
;;;
;;; End-of-Edit-History

;;; *STANDARD-LISP-READTABLE* is a readtable for Standard Lisp syntax.
;;; In fact, it is only slightly modified Common Lisp syntax, e.g. all
;;; Common Lisp reader macros are available, : can be used as a package
;;; qualifier.  A string of digits followed by a period reads as a
;;; decimal integer, not a float (same as Common Lisp).  This could be
;;; fixed, but not without some pain.  Also, *RAISE is not implemented.

;;; (in-package 'standard-lisp) ;;; JEA

(defvar *standard-lisp-readtable* (copy-readtable nil))

;;; Square brackets are used to read vectors.

(defun vector-reader (stream char)
  (declare (ignore char))
  char
  (coerce (read-delimited-list #\] stream t)
	  'simple-vector))

(set-macro-character #\[ #'vector-reader nil *standard-lisp-readtable*)

(set-macro-character #\]
		     (get-macro-character #\) *standard-lisp-readtable*)
		     nil
		     *standard-lisp-readtable*)

;;; Percent starts a comment.
(set-syntax-from-char #\% #\; *standard-lisp-readtable* *readtable*)

;;; Semicolon is a constituent.
(set-syntax-from-char #\; #\% *standard-lisp-readtable* *readtable*)

;;; Bang is the escape character in symbols
(set-syntax-from-char #\! #\\ *standard-lisp-readtable* *readtable*)

;;; Reading strings is a little difficult since double quotes are
;;; included in strings by using backslash in Common Lisp (equivalent
;;; to bang in Standard Lisp), but they are included by doubling in
;;; Standard Lisp (similar to Pascal's convention, for example).
;;; We simply do it the obvious way, caching a buffer to avoid consing.

(defvar *standard-lisp-string-buffer* nil)

(defun string-reader (stream char)
  (declare (ignore char))
  char
  ;;Set the buffer to NIL while we're using it, so this is re-entrant.
  (let ((string-buffer (shiftf *standard-lisp-string-buffer* nil)))
    (if (null string-buffer)
	(setq string-buffer (make-array 256
					:element-type 'string-char
					:fill-pointer 0
					:adjustable t))
	(setf (fill-pointer string-buffer) 0))
    (do ((i 0 (+ i 1)))
	(())
      (declare (fixnum i))
      (let ((ch (read-char stream t t t)))
	(when (char= ch #\")
	  ;;It's OK to be at EOF after a final string quote
	  (setq ch (read-char stream nil stream t))
	  (unless (eql ch #\")
	    (unless (eq ch stream)
	      (unread-char ch stream))
	    (return
	      (prog1 (subseq string-buffer 0 i)
		     ;;Put the buffer back after we're done with it.
		     (setq *standard-lisp-string-buffer* string-buffer)))))
	(vector-push-extend ch string-buffer)))))

(set-macro-character #\" #'string-reader nil *standard-lisp-readtable*)
